#pragma once

#include "Scriptlet.h"
#include "NPSingleton.h"
#include <atlcoll.h>
#include <atlstr.h>
#include "ScriptletConstant.h"
#include "NProtoTypeFactory.h"
#include "ScriptletDialog.h"
#include "BaseWindow.h"
#include "ActionEvent.h"
#include "WindowEvent.h"


//scriptlet container
#define DECLARE_PROPERTY_CONTAINER(CREATOR) \
	static int  sm_ContainerID;	

#define REGISTER_SCRIPTLETCONTAINER_CREATOR(CREATORID, CREATOR) \
	int CREATOR::sm_ContainerID = CScriptletContainerFactory::get_instance()->RegisterScriptContainer(CREATORID, new CNDefaultCloneable< CREATOR >);

//property accessor
#define DECLARE_PROPERTY_ACCESSOR(CREATOR) \
	static int sm_PropertyAccessorID;

#define REGISTER_PROPERTY_ACCESSOR(CREATORID, CREATOR) \
	int CREATOR::sm_PropertyAccessorID = CScriptletContainerFactory::get_instance()->RegisterPropertyAccessor(CREATORID, new CNDefaultCloneable< CREATOR >);

//component creator
#define DECLARE_COMPONENT_CREATOR(CREATOR) \
	static int sm_ComponentCreatorID;

#define REGISTER_COMPONENT_CREATOR(CREATORID, CREATOR) \
	int CREATOR::sm_ComponentCreatorID = CScriptletContainerFactory::get_instance()->RegisterComponentCreator(CREATORID, new CNDefaultCloneable< CREATOR >);

//script plugin
#define DECLARE_REGISTER_SCRIPTLETCOMPONENTPLUGIN_CREATOR(CREATOR) \
	static int sm_ScriptPluginID;

#define REGISTER_SCRIPTLETCOMPONENTPLUGIN_CREATOR(CREATORID, CREATOR) \
	int CREATOR::sm_ScriptPluginID = CScriptletContainerFactory::get_instance()->RegisterScriptPlugin(CREATORID, new CNDefaultCloneable< CREATOR >);
//datacontext
#define REGISTER_DATACONTEXT(ID, DATACONTEXT) \
	CScriptletContainerFactory::get_instance()->RegisterDataContext(ID, DATACONTEXT);
//converter
#define DECLARE_CONVERTER_CREATOR(CREATOR) \
	static int sm_ConverterID;
#define REGISTER_CONVERTER_CREATOR(CREATORID, CREATOR) \
	int CREATOR::sm_ConverterID = CScriptletContainerFactory::get_instance()->RegisterValueConveter(CREATORID, new CNDefaultCloneable< CREATOR >);

interface IItemListener;
class ComponentPropertyAccessor;
class Binding;
class PropertyBinder;
interface IDataContext;
interface IValueConverter;
class NPCOMMONUI_API CScriptletContainerFactory
{
public:
	DECLARE_SINGLETON_CODE(CScriptletContainerFactory);
	CScriptletContainerFactory(void);
	virtual ~CScriptletContainerFactory(void);

	int RegisterScriptContainer(LPCTSTR pszcID, ICloneable* pCloneable)
	{
		return m_ScriptContainerFactory.RegisterProtoType(pszcID, pCloneable);
	}
	int RegisterPropertyAccessor(LPCTSTR pszcID, ICloneable* pCloneable)
	{
		return m_ComponentPropertyAccessorFactory.RegisterProtoType(pszcID, pCloneable);
	}
	int RegisterComponentCreator(LPCTSTR pszcID, ICloneable* pCloneable)
	{
		return m_ComponentCreatorFactory.RegisterProtoType(pszcID, pCloneable);
	}
	int RegisterScriptPlugin(LPCTSTR pszcID, ICloneable* pCloneable)
	{
		return m_ScriptPluginFactory.RegisterProtoType(pszcID, pCloneable);
	}
	int RegisterDataContext(LPCTSTR pszcID, IDataContext* datacontext)
	{
		m_DataContexts.SetAt(CString(pszcID), datacontext);
		return 0;
	}
	int RegisterValueConveter(LPCTSTR pszcID, ICloneable* pCloneable)
	{
		return m_ValueConverterFactory.RegisterProtoType(pszcID, pCloneable);
	}

	//script container
	ILayoutManager* CreateLayout(LPCTSTR pszcID);
	IActionListener* CreateAction(LPCTSTR pszcID);
	IWindowListener* CreateWindowAction(LPCTSTR pszcID);
	IPropertyChangedListener* CreatePropertyChangeListener(LPCTSTR pszcID);
	IItemListener* CreateitemListener(LPCTSTR pszcID);

	//property accessor
	ComponentPropertyAccessor* CreatePropertyAccessor(LPCTSTR pszcID);

	//component creator
	CContainer* CreateScriptContainer(LPCTSTR pszcID);
	CScriptletDialog* CreateScriptDialog(LPCTSTR pszcID);
	CBaseWindow* CreateBaseWindow(LPCTSTR pszcID);

	//script plugin
	// {kr} ־ XML ؼϿ Component Ѵ. {/kr}	
	CComponent* MakeComponent(CContainer* pContainer, TiXmlElement* pNode);
	// {kr} ־ XML ؼϿ Component Ѵ. {/kr}
	BOOL GenerateComponent(CContainer* pContainer, CComponent* pComponent, TiXmlElement* pNode);
	// {kr} ־ XML ؼϿ Component ϰ Ѵ. {/kr}
	CComponent* CreateComponent(CContainer* pContainer, TiXmlElement* pNode);

	//datacontext
	IDataContext* FindDataContext(LPCTSTR pszcID);
	//converter
	IValueConverter* CreateValueConverter(LPCTSTR pszcID);

private:
	CNProtoTypeFactory m_ScriptContainerFactory;
	CNProtoTypeFactory m_ComponentPropertyAccessorFactory;
	CNProtoTypeFactory m_ComponentCreatorFactory;
	CNProtoTypeFactory m_ScriptPluginFactory;
	CNProtoTypeFactory m_ValueConverterFactory;
	
	CAtlMap<CString, IDataContext*> m_DataContexts;
};
